package com.ppopescu;

import com.ppopescu.utils.DataClass;
import com.ppopescu.utils.MyDao;
import com.ppopescu.utils.MyService;
import org.apache.logging.log4j.LogManager;
import org.junit.*;
import org.mockito.Mockito;

import java.util.Arrays;

public class Tests {
    private static MyDao mockDao;

    @BeforeClass
    public static void setup() {
        mockDao = Mockito.mock(MyDao.class);

        // write any data needed to properly test everything
    }

    @AfterClass
    public static void cleanup() {
        // Make sure that any data written is properly cleaned up at the end of the test suite
    }

    @Before
    public void beforeMethod() {
        LogManager.getLogger().info("Before method was executed");
        // Do any initializations or cleanups that are needed before all the methods in the suite
        Mockito.reset(mockDao);
    }

    @After
    public void afterMethod() {
        LogManager.getLogger().info("After method was executed");
        // Do any cleanups that are needed after each method
    }

    @Test
    public void aSimpleTest() {
        int aNumber = 5;
        String str = "A test string";
        double aDouble = 4.67;

        Assert.assertEquals(5, aNumber);
        Assert.assertEquals("A test string", str);
        Assert.assertEquals(4.67004, aDouble, 0.001);
    }

    @Test
    public void aSimpleTestWithMessage() {
        String myStr = "This test will fail";
        Assert.assertEquals("The myStr variable is not the expected value", "Nothing", myStr);
    }

    @Test
    public void testComplexObject() {
        DataClass obj1 = new DataClass("f1Value", "f2Value", 3);
        DataClass expected = new DataClass("f1Value", "f2Value", 3);

        Assert.assertEquals(expected, obj1);
        //Assert.assertSame(expected, obj1);
    }

    @Test(expected = NumberFormatException.class)
    public void testNumberParse() {
        Integer.parseInt("a");
    }

    @Test
    public void testService() {
        MyService service = new MyService();
        service.setDao(mockDao);

        Mockito.when(mockDao.getAll()).thenReturn(Arrays.asList(new DataClass("a", "b", 3)));
        Mockito.when(mockDao.get(Mockito.eq("a"))).thenReturn(new DataClass("a", "b", 4));

        boolean result = service.process("a");
        Assert.assertEquals(true, result);
    }
}
